use norun_grpc_interface::{PublishRequest, registry_service_client::RegistryServiceClient};
use tokio::sync::OnceCell;
use tonic::transport::Channel;

use crate::{project_file::ProjectFile, state::ClientState};

pub struct GrpcClient {
    url: String,
    registry_client: OnceCell<RegistryServiceClient<Channel>>,
}

impl GrpcClient {
    #[tracing::instrument(skip(self), level = "trace")]
    pub async fn publish(&self, project_file: &ProjectFile) -> anyhow::Result<()> {
        tracing::trace!("calling publish via. grpc on registry");

        let mut registry_client = self.get_registry_client().await?;

        let res = registry_client
            .publish(PublishRequest {
                project: Some(project_file.into()),
            })
            .await?;
        let _res = res.into_inner();

        Ok(())
    }

    async fn get_registry_client(&self) -> anyhow::Result<RegistryServiceClient<Channel>> {
        let client = self
            .registry_client
            .get_or_try_init(move || async move {
                let channel = Channel::from_shared(self.url.clone())?.connect().await?;
                let client = RegistryServiceClient::new(channel);

                Ok::<_, anyhow::Error>(client)
            })
            .await?;

        Ok(client.clone())
    }
}

impl From<&ProjectFile> for norun_grpc_interface::Project {
    fn from(value: &ProjectFile) -> Self {
        value.clone().into()
    }
}

impl From<ProjectFile> for norun_grpc_interface::Project {
    fn from(value: ProjectFile) -> Self {
        Self {
            name: value.project.name,
            image: value.container.image,
            version: value.container.version,
            port: value.expose.and_then(|e| e.port),
        }
    }
}

pub trait GrpcClientState {
    fn grpc_client(&self) -> GrpcClient;
}

impl GrpcClientState for ClientState {
    fn grpc_client(&self) -> GrpcClient {
        GrpcClient {
            url: self.norun_server_url.clone(),
            registry_client: OnceCell::default(),
        }
    }
}
