use std::str::FromStr;

// ProjectTag can be a destination@name:version
#[derive(Clone, Debug, PartialEq)]
pub struct ProjectTag {
    pub destination: String,
    pub name: String,
    pub version: String,
}

impl FromStr for ProjectTag {
    type Err = anyhow::Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let (destination, rest) = s.split_once('@').ok_or(anyhow::anyhow!("failed to find an '@' in your project tag, make sure it looks like 'destination@name:version'"))?;
        if destination.is_empty() {
            anyhow::bail!("destination is required like so 'destination@name:version'");
        }

        let (name, version) = rest.split_once(':').ok_or(anyhow::anyhow!("failed to find an ':' in your project tag, make sure it looks like 'destination@name:version'"))?;
        if name.is_empty() {
            anyhow::bail!("name is required like so 'destination@name:version'");
        }
        if version.is_empty() {
            anyhow::bail!("version is required like so 'destination@name:version'");
        }

        Ok(Self {
            destination: destination.to_string(),
            name: name.to_string(),
            version: version.to_string(),
        })
    }
}

#[cfg(test)]
mod test {
    use std::str::FromStr;

    use crate::models::ProjectTag;

    #[test]
    fn happy_path() -> anyhow::Result<()> {
        let raw = "destination@name:version";
        let expected = ProjectTag {
            destination: "destination".to_string(),
            name: "name".to_string(),
            version: "version".to_string(),
        };

        let actual = ProjectTag::from_str(raw)?;

        pretty_assertions::assert_eq!(expected, actual);

        Ok(())
    }

    #[test]
    fn missing_destination_fails() -> anyhow::Result<()> {
        let raw = "@name:version";

        ProjectTag::from_str(raw).expect_err("test expected err");

        Ok(())
    }

    #[test]
    fn missing_destination_tag_fails() -> anyhow::Result<()> {
        let raw = "name:version";

        ProjectTag::from_str(raw).expect_err("test expected err");

        Ok(())
    }

    #[test]
    fn missing_name_fails() -> anyhow::Result<()> {
        let raw = "destination@:version";

        ProjectTag::from_str(raw).expect_err("test expected err");

        Ok(())
    }

    #[test]
    fn missing_name_tag_fails() -> anyhow::Result<()> {
        let raw = "destination@:version";

        ProjectTag::from_str(raw).expect_err("test expected err");

        Ok(())
    }
    #[test]
    fn missing_version_fails() -> anyhow::Result<()> {
        let raw = "destination@name:";

        ProjectTag::from_str(raw).expect_err("test expected err");

        Ok(())
    }

    #[test]
    fn missing_version_tag_fails() -> anyhow::Result<()> {
        let raw = "destination@name:";

        ProjectTag::from_str(raw).expect_err("test expected err");

        Ok(())
    }
}
