use std::path::Path;

use anyhow::Context;
use serde::Deserialize;

const NORUN_PROJECT_FILE_NAME: &str = "norun.toml";

#[derive(Clone, Debug, Deserialize, PartialEq)]
pub struct ProjectFile {
    pub project: ProjectDecl,
    pub container: ContainerDecl,
    pub expose: Option<ExposeDecl>,
}

#[derive(Clone, Debug, Deserialize, PartialEq)]
pub struct ProjectDecl {
    pub name: String,
}

#[derive(Clone, Debug, Deserialize, PartialEq)]
pub struct ContainerDecl {
    pub image: String,
    pub version: String,
}

#[derive(Clone, Debug, Deserialize, PartialEq)]
pub struct ExposeDecl {
    pub port: Option<u32>,
}

pub fn parse(content: &str) -> anyhow::Result<ProjectFile> {
    let project_file: ProjectFile = toml::from_str(content)?;

    Ok(project_file)
}

pub async fn parse_file(path: &Path) -> anyhow::Result<ProjectFile> {
    let path = if let Some(file_name) = path.file_name()
        && file_name == NORUN_PROJECT_FILE_NAME
    {
        path
    } else {
        &path.join(NORUN_PROJECT_FILE_NAME)
    };

    let file = tokio::fs::read_to_string(path)
        .await
        .context("failed to read project file in the given directory")?;

    parse(&file)
}

#[cfg(test)]
mod test {
    use crate::project_file::*;

    #[test]
    fn can_parse_file_happy_path() -> anyhow::Result<()> {
        let raw = r#"
[project]
name = "hello-world"

[container]
image = "kasperhermansen/hello-world"
version = "latest" # default

[expose]
port = 8080
        "#;

        let expected = ProjectFile {
            project: ProjectDecl {
                name: "hello-world".into(),
            },
            container: ContainerDecl {
                image: "kasperhermansen/hello-world".into(),
                version: "latest".into(),
            },
            expose: Some(ExposeDecl { port: Some(8080) }),
        };

        let actual = parse(raw)?;

        pretty_assertions::assert_eq!(expected, actual);

        Ok(())
    }
}
